@@ -20,13 +20,15 @@ set -e
 usage() {
     echo
     echo "Usage: $(basename $0)"
-    echo "            [-a i386 | **amd64** | armhf ]"
+    echo "            [-a i386 | **amd64** | armhf | ppc64le]"
     echo "            [-b **haproxy** ]"
     echo "            [-c **~/.cache/image-create** | <cache directory> ]"
-    echo "            [-d **xenial**/**7** | trusty | <other release id> ]"
+    echo "            [-d **bionic**/**7** | xenial | 8 | <other release id> ]"
     echo "            [-e]"
+    echo "            [-f]"
     echo "            [-h]"
-    echo "            [-i **ubuntu-minimal** | fedora | centos | rhel ]"
+    echo "            [-i **ubuntu-minimal** | fedora | centos-minimal | rhel ]"
+    echo "            [-k <kernel package name> ]"
     echo "            [-l <log file> ]"
     echo "            [-n]"
     echo "            [-o **amphora-x64-haproxy** | <filename> ]"
@@ -41,10 +43,12 @@ usage() {
     echo "        '-a' is the architecture type for the image (default: amd64)"
     echo "        '-b' is the backend type (default: haproxy)"
     echo "        '-c' is the path to the cache directory (default: ~/.cache/image-create)"
-    echo "        '-d' distribution release id (default on ubuntu: xenial)"
+    echo "        '-d' distribution release id (default on ubuntu: bionic)"
     echo "        '-e' enable complete mandatory access control systems when available (default: permissive)"
+    echo "        '-f' disable tmpfs for build"
     echo "        '-h' display this help message"
-    echo "        '-i' is the base OS (default: ubuntu)"
+    echo "        '-i' is the base OS (default: ubuntu-minimal)"
+    echo "        '-k' is the kernel meta package name, currently only for ubuntu-minimal base OS (default: linux-image-virtual)"
     echo "        '-l' is output logfile (default: none)"
     echo "        '-n' disable sshd (default: enabled)"
     echo "        '-o' is the output image file name"
@@ -85,12 +89,13 @@ dib_enable_tracing=
 
 AMP_LOGFILE=""
 
-while getopts "a:b:c:d:ehi:l:no:pt:r:s:vw:x" opt; do
+while getopts "a:b:c:d:efhi:k:l:no:pt:r:s:vw:x" opt; do
     case $opt in
         a)
             AMP_ARCH=$OPTARG
             if [ $AMP_ARCH != "i386" ] && \
                 [ $AMP_ARCH != "amd64" ] && \
+                [ $AMP_ARCH != "ppc64le" ] && \
                 [ $AMP_ARCH != "armhf" ]; then
                 echo "Error: Unsupported architecture " $AMP_ARCH " specified"
                 exit 3
@@ -113,6 +118,9 @@ while getopts "a:b:c:d:ehi:l:no:pt:r:s:vw:x" opt; do
         e)
             AMP_ENABLE_FULL_MAC_SECURITY=1
         ;;
+        f)
+            AMP_DISABLE_TMP_FS='--no-tmpfs'
+        ;;
         h)
             usage
         ;;
@@ -122,6 +130,7 @@ while getopts "a:b:c:d:ehi:l:no:pt:r:s:vw:x" opt; do
                 [ $AMP_BASEOS != "ubuntu-minimal" ] && \
                 [ $AMP_BASEOS != "fedora" ] && \
                 [ $AMP_BASEOS != "centos" ] && \
+                [ $AMP_BASEOS != "centos-minimal" ] && \
                 [ $AMP_BASEOS != "rhel" ]; then
                 echo "Error: Unsupported base OS " $AMP_BASEOS " specified"
                 exit 3
@@ -129,6 +138,12 @@ while getopts "a:b:c:d:ehi:l:no:pt:r:s:vw:x" opt; do
             if [ $AMP_BASEOS == "ubuntu" ]; then
                 AMP_BASEOS="ubuntu-minimal"
             fi
+            if [ $AMP_BASEOS == "centos" ]; then
+                AMP_BASEOS="centos-minimal"
+            fi
+        ;;
+        k)
+            AMP_KERNEL=$OPTARG
         ;;
         l)
             AMP_LOGFILE="--logfile=$OPTARG"
@@ -138,7 +153,7 @@ while getopts "a:b:c:d:ehi:l:no:pt:r:s:vw:x" opt; do
         ;;
         o)
             AMP_OUTPUTFILENAME=$(readlink -f $OPTARG)
-	    amp_dir=$(dirname $AMP_OUTPUTFILENAME)
+            amp_dir=$(dirname $AMP_OUTPUTFILENAME)
             if [ ! -d $amp_dir ]; then
                 echo "Error: Directory $amp_dir does not exist"
                 exit 3
@@ -192,12 +207,15 @@ AMP_ARCH=${AMP_ARCH:-"amd64"}
 AMP_BACKEND=${AMP_BACKEND:-"haproxy-octavia"}
 
 AMP_CACHEDIR=${AMP_CACHEDIR:-"$HOME/.cache/image-create"}
+# Make sure we have an absolute path for the cache location
+mkdir -p $AMP_CACHEDIR
+AMP_CACHEDIR="$( cd "$AMP_CACHEDIR" && pwd )"
 
 AMP_BASEOS=${AMP_BASEOS:-"ubuntu-minimal"}
 
 if [ "$AMP_BASEOS" = "ubuntu-minimal" ]; then
-    export DIB_RELEASE=${AMP_DIB_RELEASE:-"xenial"}
-elif [ "${AMP_BASEOS}" = "centos" ] || [ "${AMP_BASEOS}" = "rhel" ]; then
+    export DIB_RELEASE=${AMP_DIB_RELEASE:-"bionic"}
+elif [ "${AMP_BASEOS}" = "centos-minimal" ] || [ "${AMP_BASEOS}" = "rhel" ]; then
     export DIB_RELEASE=${AMP_DIB_RELEASE:-"7"}
 elif [ "${AMP_BASEOS}" = "fedora" ]; then
     export DIB_RELEASE=${AMP_DIB_RELEASE:-"28"}
@@ -209,14 +227,20 @@ AMP_IMAGETYPE=${AMP_IMAGETYPE:-"qcow2"}
 
 AMP_IMAGESIZE=${AMP_IMAGESIZE:-2}
 
+if [ "$AMP_BASEOS" = "ubuntu-minimal" ]; then
+    export DIB_UBUNTU_KERNEL=${AMP_KERNEL:-"linux-image-virtual"}
+fi
+
 AMP_DISABLE_SSHD=${AMP_DISABLE_SSHD:-0}
 
 AMP_PACKAGE_INSTALL=${AMP_PACKAGE_INSTALL:-0}
 
 AMP_ENABLE_FULL_MAC_SECURITY=${AMP_ENABLE_FULL_MAC_SECURITY:-0}
 
-if [ "$AMP_BASEOS" = "rhel" -o "$AMP_BASEOS" = "centos" -o "$AMP_BASEOS" = "fedora" ] && [ "$AMP_IMAGESIZE" -lt 3 ]; then
-    echo "RHEL/centos based amphora requires an image size of at least 3GB"
+AMP_DISABLE_TMP_FS=${AMP_DISABLE_TMP_FS:-""}
+
+if [ "$AMP_BASEOS" = "rhel" -o "$AMP_BASEOS" = "centos-minimal" -o "$AMP_BASEOS" = "fedora" ] && [ "$AMP_IMAGESIZE" -lt 3 ]; then
+    echo "RHEL/CentOS based amphora requires an image size of at least 3GB"
     exit 1
 fi
 
@@ -254,8 +278,8 @@ export CLOUD_INIT_DATASOURCES=${CLOUD_INIT_DATASOURCES:-"ConfigDrive"}
 # Additional RHEL environment checks
 if [ "${AMP_BASEOS}" = "rhel" ]; then
     if [ -z "${DIB_LOCAL_IMAGE}" ]; then
-        echo "DIB_LOCAL_IMAGE variable must be set and point to a RHEL 7 base cloud image. Exiting."
-        echo "For more information, see the README file in ${DIB_ELEMENTS_PATH}/elements/rhel7"
+        echo "DIB_LOCAL_IMAGE variable must be set and point to a RHEL base cloud image. Exiting."
+        echo "For more information, see the README file in ${DIB_ELEMENTS_PATH}/elements/rhel"
         exit 1
     fi
 fi
@@ -284,10 +308,11 @@ fi
 
 # Make sure we have the required packages installed
 if [ "$platform" = 'NAME="Ubuntu"' ]; then
-    PKG_LIST="qemu git"
+    PKG_LIST="qemu-utils git kpartx debootstrap"
     for pkg in $PKG_LIST; do
-        if ! dpkg --get-selections | grep -q "^$pkg[[:space:]]*install$" >/dev/null; then
+        if ! dpkg --get-selections 2> /dev/null | grep -q "^$pkg[[:space:]]*install$" >/dev/null; then
             echo "Required package " $pkg " is not installed.  Exiting."
+            echo "Binary dependencies on this platform are: ${PKG_LIST}"
             exit 1
         fi
     done
@@ -307,6 +332,7 @@ elif [[ $platform =~ "SUSE" ]]; then
     for pkg in $PKG_LIST; do
         if ! rpm -q $pkg &> /dev/null; then
             echo "Required package " ${pkg/\*} " is not installed.  Exiting."
+            echo "Binary dependencies on this platform are: ${PKG_LIST}"
             exit 1
         fi
     done
@@ -318,6 +344,7 @@ else
     for pkg in $PKG_LIST; do
         if ! rpm -qa $pkg ; then
             echo "Required package " ${pkg/\*} " is not installed.  Exiting."
+            echo "Binary dependencies on this platform are: ${PKG_LIST}"
             exit 1
         fi
     done
@@ -334,11 +361,10 @@ pushd $TEMP > /dev/null
 # Setup the elements list
 
 AMP_element_sequence=${AMP_element_sequence:-"base vm"}
-if [ "${AMP_BASEOS}" = "centos" ] || [ "${AMP_BASEOS}" = "rhel" ]; then
-    AMP_element_sequence="$AMP_element_sequence ${AMP_BASEOS}${DIB_RELEASE}"
-else
-    AMP_element_sequence="$AMP_element_sequence ${AMP_BASEOS}"
+if [ "${AMP_BASEOS}" = "rhel" ] && [ "${DIB_RELEASE}" = "8" ]; then
+    export DIB_INSTALLTYPE_pip_and_virtualenv=package
 fi
+AMP_element_sequence="$AMP_element_sequence ${AMP_BASEOS}"
 
 if [ "$AMP_PACKAGE_INSTALL" -eq 1 ]; then
     export DIB_INSTALLTYPE_amphora_agent=package
@@ -359,14 +385,24 @@ AMP_element_sequence="$AMP_element_sequence rebind-sshd"
 AMP_element_sequence="$AMP_element_sequence no-resolvconf"
 AMP_element_sequence="$AMP_element_sequence amphora-agent"
 AMP_element_sequence="$AMP_element_sequence sos"
+AMP_element_sequence="$AMP_element_sequence cloud-init-datasources"
 
 if [ "$AMP_ENABLE_FULL_MAC_SECURITY" -ne 1 ]; then
     # SELinux systems
-    if [ "${AMP_BASEOS}" = "centos" ] || [ "${AMP_BASEOS}" = "fedora" ] || [ "${AMP_BASEOS}" = "rhel" ]; then
+    if [ "${AMP_BASEOS}" = "centos-minimal" ] || [ "${AMP_BASEOS}" = "fedora" ] || [ "${AMP_BASEOS}" = "rhel" ]; then
         AMP_element_sequence="$AMP_element_sequence selinux-permissive"
     fi
 fi
 
+# Disable the dnf makecache timer
+if [ "${AMP_BASEOS}" = "centos-minimal" ] || [ "${AMP_BASEOS}" = "fedora" ] || [ "${AMP_BASEOS}" = "rhel" ]; then
+    AMP_element_sequence="$AMP_element_sequence disable-makecache"
+fi
+
+if [ "${AMP_BASEOS}" = "centos-minimal" ]; then
+    export DIB_YUM_MINIMAL_CREATE_INTERFACES=0
+fi
+
 # Add keepalived-octavia element
 AMP_element_sequence="$AMP_element_sequence keepalived-octavia"
 AMP_element_sequence="$AMP_element_sequence ipvsadmin"
@@ -391,21 +427,26 @@ if [ "$DIB_LOCAL_ELEMENTS" ]; then
     AMP_element_sequence="$AMP_element_sequence $DIB_LOCAL_ELEMENTS"
 fi
 
-# Set Grub timeout to 0 (no timeout) for fast boot times
-export DIB_GRUB_TIMEOUT=${DIB_GRUB_TIMEOUT:-0}
-
 # Build the image
 
-if [ "$AMP_BASEOS" = "ubuntu-minimal" ]; then
-    export DIB_CLOUD_INIT_DATASOURCES=$CLOUD_INIT_DATASOURCES
-fi
+export DIB_CLOUD_INIT_DATASOURCES=$CLOUD_INIT_DATASOURCES
 
 dib_trace_arg=
 if [ -n "$dib_enable_tracing" ]; then
     dib_trace_arg="-x"
 fi
 
-disk-image-create $AMP_LOGFILE $dib_trace_arg -a $AMP_ARCH -o $AMP_OUTPUTFILENAME -t $AMP_IMAGETYPE --image-size $AMP_IMAGESIZE --image-cache $AMP_CACHEDIR $AMP_element_sequence
+if [ "$USE_PYTHON3" = "False" ]; then
+    export DIB_PYTHON_VERSION=2
+fi
+
+disk-image-create $AMP_LOGFILE $dib_trace_arg -a $AMP_ARCH -o $AMP_OUTPUTFILENAME -t $AMP_IMAGETYPE --image-size $AMP_IMAGESIZE --image-cache $AMP_CACHEDIR $AMP_DISABLE_TMP_FS $AMP_element_sequence
 
 popd > /dev/null # out of $TEMP
 rm -rf $TEMP
+
+if [ -z "$DIB_REPOREF_amphora_agent" ]; then
+    echo "Successfully built the amphora image using amphora-agent from the master branch."
+else
+    echo "Successfully built the amphora using the $DIB_REPOREF_amphora_agent amphora-agent."
+fi
