@@ -91,7 +91,8 @@ class HaproxyAmphoraLoadBalancerDriver(
 
         return haproxy_version_string.split('.')[:2]
 
-    def _populate_amphora_api_version(self, amphora):
+    def _populate_amphora_api_version(self, amphora, timeout_dict=None,
+                                      raise_retry_exception=False):
         """Populate the amphora object with the api_version
 
         This will query the amphora for version discovery and populate
@@ -102,7 +103,8 @@ class HaproxyAmphoraLoadBalancerDriver(
         if not getattr(amphora, 'api_version', None):
             try:
                 amphora.api_version = self.clients['base'].get_api_version(
-                    amphora)['api_version']
+                    amphora, timeout_dict=timeout_dict,
+                    raise_retry_exception=raise_retry_exception)['api_version']
             except exc.NotFound:
                 # Amphora is too old for version discovery, default to 0.5
                 amphora.api_version = '0.5'
@@ -290,8 +292,11 @@ class HaproxyAmphoraLoadBalancerDriver(
                         getattr(self.clients[amp.api_version], func_name)(
                             amp, loadbalancer.id, *args)
 
-    def start(self, loadbalancer, amphora=None):
-        self._apply('start_listener', loadbalancer, amphora)
+    def reload(self, loadbalancer, amphora=None, timeout_dict=None):
+        self._apply('reload_listener', loadbalancer, amphora, timeout_dict)
+
+    def start(self, loadbalancer, amphora=None, timeout_dict=None):
+        self._apply('start_listener', loadbalancer, amphora, timeout_dict)
 
     def delete(self, listener):
         # Delete any UDP listeners the old way (we didn't update the way they
@@ -364,9 +369,11 @@ class HaproxyAmphoraLoadBalancerDriver(
             self.clients[amphora.api_version].delete_listener(
                 amphora, listener.load_balancer.id)
 
-    def get_info(self, amphora):
-        self._populate_amphora_api_version(amphora)
-        return self.clients[amphora.api_version].get_info(amphora)
+    def get_info(self, amphora, raise_retry_exception=False):
+        self._populate_amphora_api_version(
+            amphora, raise_retry_exception=raise_retry_exception)
+        return self.clients[amphora.api_version].get_info(
+            amphora, raise_retry_exception=raise_retry_exception)
 
     def get_diagnostics(self, amphora):
         pass
@@ -577,6 +584,28 @@ class HaproxyAmphoraLoadBalancerDriver(
                       'API.'.format(amphora.id))
             raise driver_except.AmpDriverNotImplementedError()
 
+    def get_interface_from_ip(self, amphora, ip_address, timeout_dict=None):
+        """Get the interface name for an IP address.
+
+        :param amphora: The amphora to query.
+        :type amphora: octavia.db.models.Amphora
+        :param ip_address: The IP address to lookup. (IPv4 or IPv6)
+        :type ip_address: string
+        :param timeout_dict: Dictionary of timeout values for calls to the
+                             amphora. May contain: req_conn_timeout,
+                             req_read_timeout, conn_max_retries,
+                             conn_retry_interval
+        :type timeout_dict: dict
+        :returns: None if not found, the interface name string if found.
+        """
+        try:
+            self._populate_amphora_api_version(amphora, timeout_dict)
+            response_json = self.clients[amphora.api_version].get_interface(
+                amphora, ip_address, timeout_dict, log_error=False)
+            return response_json.get('interface', None)
+        except (exc.NotFound, driver_except.TimeOutException):
+            return None
+
 
 # Check a custom hostname
 class CustomHostNameCheckingAdapter(requests.adapters.HTTPAdapter):
@@ -624,7 +653,7 @@ class AmphoraAPIClientBase(object):
             port=CONF.haproxy_amphora.bind_port)
 
     def request(self, method, amp, path='/', timeout_dict=None,
-                retry_404=True, **kwargs):
+                retry_404=True, raise_retry_exception=False, **kwargs):
         cfg_ha_amp = CONF.haproxy_amphora
         if timeout_dict is None:
             timeout_dict = {}
@@ -689,7 +718,13 @@ class AmphoraAPIClientBase(object):
                 exception = e
                 LOG.warning("Could not connect to instance. Retrying.")
                 time.sleep(conn_retry_interval)
-
+                if raise_retry_exception:
+                    # For taskflow persistence cause attribute should
+                    # be serializable to JSON. Pass None, as cause exception
+                    # is described in the expection message.
+                    six.raise_from(
+                        driver_except.AmpConnectionRetry(exception=str(e)),
+                        None)
         LOG.error("Connection retries (currently set to %(max_retries)s) "
                   "exhausted.  The amphora is unavailable. Reason: "
                   "%(exception)s",
@@ -697,9 +732,11 @@ class AmphoraAPIClientBase(object):
                    'exception': exception})
         raise driver_except.TimeOutException()
 
-    def get_api_version(self, amp):
+    def get_api_version(self, amp, timeout_dict=None,
+                        raise_retry_exception=False):
         amp.api_version = None
-        r = self.get(amp, retry_404=False)
+        r = self.get(amp, retry_404=False, timeout_dict=timeout_dict,
+                     raise_retry_exception=raise_retry_exception)
         # Handle 404 special as we don't want to log an ERROR on 404
         exc.check_exception(r, (404,))
         if r.status_code == 404:
@@ -766,8 +803,8 @@ class AmphoraAPIClient0_5(AmphoraAPIClientBase):
             amp, 'listeners/{listener_id}'.format(listener_id=listener_id))
         return exc.check_exception(r, (404,))
 
-    def get_info(self, amp):
-        r = self.get(amp, "info")
+    def get_info(self, amp, raise_retry_exception=False):
+        r = self.get(amp, "info", raise_retry_exception=raise_retry_exception)
         if exc.check_exception(r):
             return r.json()
         return None
@@ -799,16 +836,15 @@ class AmphoraAPIClient0_5(AmphoraAPIClientBase):
         r = self.put(amp, 'vrrp/upload', data=config)
         return exc.check_exception(r)
 
-    def _vrrp_action(self, action, amp):
-        r = self.put(amp, 'vrrp/{action}'.format(action=action))
+    def _vrrp_action(self, action, amp, timeout_dict=None):
+        r = self.put(amp, 'vrrp/{action}'.format(action=action),
+                     timeout_dict=timeout_dict)
         return exc.check_exception(r)
 
-    def get_interface(self, amp, ip_addr, timeout_dict=None):
+    def get_interface(self, amp, ip_addr, timeout_dict=None, log_error=True):
         r = self.get(amp, 'interface/{ip_addr}'.format(ip_addr=ip_addr),
                      timeout_dict=timeout_dict)
-        if exc.check_exception(r):
-            return r.json()
-        return None
+        return exc.check_exception(r, log_error=log_error).json()
 
     def upload_udp_config(self, amp, listener_id, config, timeout_dict=None):
         r = self.put(
@@ -896,8 +932,8 @@ class AmphoraAPIClient1_0(AmphoraAPIClientBase):
             amp, 'listeners/{object_id}'.format(object_id=object_id))
         return exc.check_exception(r, (404,))
 
-    def get_info(self, amp):
-        r = self.get(amp, "info")
+    def get_info(self, amp, raise_retry_exception=False):
+        r = self.get(amp, "info", raise_retry_exception=raise_retry_exception)
         if exc.check_exception(r):
             return r.json()
         return None
@@ -929,16 +965,15 @@ class AmphoraAPIClient1_0(AmphoraAPIClientBase):
         r = self.put(amp, 'vrrp/upload', data=config)
         return exc.check_exception(r)
 
-    def _vrrp_action(self, action, amp):
-        r = self.put(amp, 'vrrp/{action}'.format(action=action))
+    def _vrrp_action(self, action, amp, timeout_dict=None):
+        r = self.put(amp, 'vrrp/{action}'.format(action=action),
+                     timeout_dict=timeout_dict)
         return exc.check_exception(r)
 
-    def get_interface(self, amp, ip_addr, timeout_dict=None):
+    def get_interface(self, amp, ip_addr, timeout_dict=None, log_error=True):
         r = self.get(amp, 'interface/{ip_addr}'.format(ip_addr=ip_addr),
                      timeout_dict=timeout_dict)
-        if exc.check_exception(r):
-            return r.json()
-        return None
+        return exc.check_exception(r, log_error=log_error).json()
 
     def upload_udp_config(self, amp, listener_id, config, timeout_dict=None):
         r = self.put(
