@@ -307,3 +307,15 @@ class BaseController(pecan.rest.RestController):
                 raise exceptions.ValidationException(detail=_(
                     "The CRL specified is not valid for client certificate "
                     "authority reference supplied."))
+
+    @staticmethod
+    def _validate_protocol(listener_protocol, pool_protocol):
+        proto_map = constants.VALID_LISTENER_POOL_PROTOCOL_MAP
+        for valid_pool_proto in proto_map[listener_protocol]:
+            if pool_protocol == valid_pool_proto:
+                return
+        detail = _("The pool protocol '%(pool_protocol)s' is invalid while "
+                   "the listener protocol is '%(listener_protocol)s'.") % {
+                       "pool_protocol": pool_protocol,
+                       "listener_protocol": listener_protocol}
+        raise exceptions.ValidationException(detail=detail)
