@@ -294,7 +294,10 @@ SUPPORTED_TASKFLOW_ENGINE_TYPES = ['serial', 'parallel']
 # Task/Flow constants
 ACTIVE_CONNECTIONS = 'active_connections'
 ADDED_PORTS = 'added_ports'
+ADMIN_STATE_UP = 'admin_state_up'
+ALLOWED_ADDRESS_PAIRS = 'allowed_address_pairs'
 AMP_DATA = 'amp_data'
+AMP_VRRP_INT = 'amp_vrrp_int'
 AMPHORA = 'amphora'
 AMPHORA_ID = 'amphora_id'
 AMPHORA_INDEX = 'amphora_index'
@@ -303,9 +306,12 @@ AMPHORAE = 'amphorae'
 AMPHORAE_NETWORK_CONFIG = 'amphorae_network_config'
 AMPS_DATA = 'amps_data'
 ANTI_AFFINITY = 'anti-affinity'
+ATTEMPT_NUMBER = 'attempt_number'
+BASE_PORT = 'base_port'
 BYTES_IN = 'bytes_in'
 BYTES_OUT = 'bytes_out'
 CA_TLS_CERTIFICATE_ID = 'ca_tls_certificate_id'
+CIDR = 'cidr'
 CLIENT_CA_TLS_CERTIFICATE_ID = 'client_ca_tls_certificate_id'
 CLIENT_CRL_CONTAINER_ID = 'client_crl_container_id'
 COMPUTE_ID = 'compute_id'
@@ -317,17 +323,22 @@ CRL_CONTAINER_ID = 'crl_container_id'
 DELTA = 'delta'
 DELTAS = 'deltas'
 DESCRIPTION = 'description'
+DEVICE_OWNER = 'device_owner'
 ENABLED = 'enabled'
+FAILED_AMP_VRRP_PORT_ID = 'failed_amp_vrrp_port_id'
 FAILED_AMPHORA = 'failed_amphora'
 FAILOVER_AMPHORA = 'failover_amphora'
 FAILOVER_AMPHORA_ID = 'failover_amphora_id'
 FIELDS = 'fields'
+FIXED_IPS = 'fixed_ips'
 FLAVOR_ID = 'flavor_id'
 HEALTH_MON = 'health_mon'
 HEALTH_MONITOR = 'health_monitor'
 HEALTH_MONITOR_ID = 'health_monitor_id'
 HEALTH_MONITOR_UPDATES = 'health_monitor_updates'
+ID = 'id'
 IP_ADDRESS = 'ip_address'
+IPV6_ICMP = 'ipv6-icmp'
 L7POLICY = 'l7policy'
 L7POLICY_ID = 'l7policy_id'
 L7POLICY_UPDATES = 'l7policy_updates'
@@ -345,17 +356,21 @@ MEMBER = 'member'
 MEMBER_ID = 'member_id'
 MEMBER_PORTS = 'member_ports'
 MEMBER_UPDATES = 'member_updates'
+MESSAGE = 'message'
 NAME = 'name'
 NETWORK_ID = 'network_id'
 NICS = 'nics'
 OBJECT = 'object'
+PASSIVE_FAILURE = 'passive_failure'
 PEER_PORT = 'peer_port'
 POOL = 'pool'
 POOL_CHILD_COUNT = 'pool_child_count'
 POOL_ID = 'pool_id'
 POOL_UPDATES = 'pool_updates'
+PORT = 'port'
 PORT_ID = 'port_id'
 PORTS = 'ports'
+PROJECT_ID = 'project_id'
 PROVIDER = 'provider'
 PROVIDER_NAME = 'provider_name'
 QOS_POLICY_ID = 'qos_policy_id'
@@ -363,12 +378,18 @@ REDIRECT_POOL = 'redirect_pool'
 REQ_CONN_TIMEOUT = 'req_conn_timeout'
 REQ_READ_TIMEOUT = 'req_read_timeout'
 REQUEST_ERRORS = 'request_errors'
+SECURITY_GROUPS = 'security_groups'
+SECURITY_GROUP_RULES = 'security_group_rules'
 SERVER_GROUP_ID = 'server_group_id'
 SERVER_PEM = 'server_pem'
 SNI_CONTAINERS = 'sni_containers'
 SOFT_ANTI_AFFINITY = 'soft-anti-affinity'
+STATUS_CODE = 'status_code'
+SUBNET = 'subnet'
+SUBNET_ID = 'subnet_id'
 SUBNET = 'subnet'
 TAGS = 'tags'
+TENANT_ID = 'tenant_id'
 TIMEOUT_DICT = 'timeout_dict'
 TLS_CERTIFICATE_ID = 'tls_certificate_id'
 TLS_CONTAINER_ID = 'tls_container_id'
@@ -377,12 +398,17 @@ TOTAL_CONNECTIONS = 'total_connections'
 UPDATED_AT = 'updated_at'
 UPDATE_DICT = 'update_dict'
 VIP = 'vip'
+VIP_ADDRESS = 'vip_address'
 VIP_NETWORK = 'vip_network'
+VIP_SG_ID = 'vip_sg_id'
+VIP_SUBNET = 'vip_subnet'
 VRRP_GROUP = 'vrrp_group'
+VRRP_PORT = 'vrrp_port'
 
 # Taskflow flow and task names
 CERT_ROTATE_AMPHORA_FLOW = 'octavia-cert-rotate-amphora-flow'
 CREATE_AMPHORA_FLOW = 'octavia-create-amphora-flow'
+CREATE_AMPHORA_RETRY_SUBFLOW = 'octavia-create-amphora-retry-subflow'
 CREATE_AMPHORA_FOR_LB_FLOW = 'octavia-create-amp-for-lb-flow'
 CREATE_HEALTH_MONITOR_FLOW = 'octavia-create-health-monitor-flow'
 CREATE_LISTENER_FLOW = 'octavia-create-listener_flow'
@@ -397,6 +423,7 @@ CREATE_POOL_FLOW = 'octavia-create-pool-flow'
 CREATE_L7POLICY_FLOW = 'octavia-create-l7policy-flow'
 CREATE_L7RULE_FLOW = 'octavia-create-l7rule-flow'
 DELETE_AMPHORA_FLOW = 'octavia-delete-amphora-flow'
+DELETE_EXTRA_AMPHORAE_FLOW = 'octavia-delete-extra-amphorae-flow'
 DELETE_HEALTH_MONITOR_FLOW = 'octavia-delete-health-monitor-flow'
 DELETE_LISTENER_FLOW = 'octavia-delete-listener_flow'
 DELETE_LOADBALANCER_FLOW = 'octavia-delete-loadbalancer-flow'
@@ -405,6 +432,7 @@ DELETE_POOL_FLOW = 'octavia-delete-pool-flow'
 DELETE_L7POLICY_FLOW = 'octavia-delete-l7policy-flow'
 DELETE_L7RULE_FLOW = 'octavia-delete-l7policy-flow'
 FAILOVER_AMPHORA_FLOW = 'octavia-failover-amphora-flow'
+FAILOVER_LOADBALANCER_FLOW = 'octavia-failover-loadbalancer-flow'
 LOADBALANCER_NETWORKING_SUBFLOW = 'octavia-new-loadbalancer-net-subflow'
 UPDATE_HEALTH_MONITOR_FLOW = 'octavia-update-health-monitor-flow'
 UPDATE_LISTENER_FLOW = 'octavia-update-listener-flow'
@@ -418,10 +446,13 @@ UPDATE_AMPHORA_CONFIG_FLOW = 'octavia-update-amp-config-flow'
 
 POST_MAP_AMP_TO_LB_SUBFLOW = 'octavia-post-map-amp-to-lb-subflow'
 CREATE_AMP_FOR_LB_SUBFLOW = 'octavia-create-amp-for-lb-subflow'
+CREATE_AMP_FOR_FAILOVER_SUBFLOW = 'octavia-create-amp-for-failover-subflow'
 AMP_PLUG_NET_SUBFLOW = 'octavia-plug-net-subflow'
 GET_AMPHORA_FOR_LB_SUBFLOW = 'octavia-get-amphora-for-lb-subflow'
 POST_LB_AMP_ASSOCIATION_SUBFLOW = (
     'octavia-post-loadbalancer-amp_association-subflow')
+AMPHORA_LISTENER_START_SUBFLOW = 'amphora-listener-start-subflow'
+AMPHORA_LISTENER_RELOAD_SUBFLOW = 'amphora-listener-start-subflow'
 
 MAP_LOADBALANCER_TO_AMPHORA = 'octavia-mapload-balancer-to-amphora'
 RELOAD_AMPHORA = 'octavia-reload-amphora'
@@ -437,7 +468,7 @@ COMPUTE_WAIT = 'octavia-compute-wait'
 UPDATE_AMPHORA_INFO = 'octavia-update-amphora-info'
 AMPHORA_FINALIZE = 'octavia-amphora-finalize'
 MARK_AMPHORA_ALLOCATED_INDB = 'octavia-mark-amphora-allocated-indb'
-RELOADLOAD_BALANCER = 'octavia-reloadload-balancer'
+MARK_AMPHORA_READY_INDB = 'octavia-mark-amphora-ready-indb'
 MARK_LB_ACTIVE_INDB = 'octavia-mark-lb-active-indb'
 MARK_AMP_MASTER_INDB = 'octavia-mark-amp-master-indb'
 MARK_AMP_BACKUP_INDB = 'octavia-mark-amp-backup-indb'
@@ -451,6 +482,7 @@ CREATE_VRRP_GROUP_FOR_LB = 'octavia-create-vrrp-group-for-lb'
 CREATE_VRRP_SECURITY_RULES = 'octavia-create-vrrp-security-rules'
 AMP_COMPUTE_CONNECTIVITY_WAIT = 'octavia-amp-compute-connectivity-wait'
 AMP_LISTENER_UPDATE = 'octavia-amp-listeners-update'
+AMP_LISTENER_START = 'octavia-amp-listeners-start'
 PLUG_VIP_AMPHORA = 'octavia-amp-plug-vip'
 APPLY_QOS_AMP = 'octavia-amp-apply-qos'
 UPDATE_AMPHORA_VIP_DATA = 'ocatvia-amp-update-vip-data'
@@ -458,6 +490,8 @@ GET_AMP_NETWORK_CONFIG = 'octavia-amp-get-network-config'
 AMP_POST_VIP_PLUG = 'octavia-amp-post-vip-plug'
 GENERATE_SERVER_PEM_TASK = 'GenerateServerPEMTask'
 AMPHORA_CONFIG_UPDATE_TASK = 'AmphoraConfigUpdateTask'
+FIRST_AMP_NETWORK_CONFIGS = 'first-amp-network-configs'
+FIRST_AMP_VRRP_INTERFACE = 'first-amp-vrrp_interface'
 
 # Batch Member Update constants
 UNORDERED_MEMBER_UPDATES_FLOW = 'octavia-unordered-member-updates-flow'
@@ -472,11 +506,30 @@ UPDATE_MEMBER_INDB = 'octavia-update-member-indb'
 DELETE_MEMBER_INDB = 'octavia-delete-member-indb'
 
 # Task Names
+ADMIN_DOWN_PORT = 'admin-down-port'
+AMPHORA_POST_VIP_PLUG = 'amphora-post-vip-plug'
+AMPHORA_RELOAD_LISTENER = 'amphora-reload-listener'
+AMPHORA_TO_ERROR_ON_REVERT = 'amphora-to-error-on-revert'
+AMPHORAE_POST_NETWORK_PLUG = 'amphorae-post-network-plug'
+ATTACH_PORT = 'attach-port'
+CALCULATE_AMPHORA_DELTA = 'calculate-amphora-delta'
+CREATE_VIP_BASE_PORT = 'create-vip-base-port'
+DELETE_AMPHORA = 'delete-amphora'
+DELETE_PORT = 'delete-port'
+DISABLE_AMP_HEALTH_MONITORING = 'disable-amphora-health-monitoring'
+GET_AMPHORA_NETWORK_CONFIGS_BY_ID = 'get-amphora-network-configs-by-id'
+GET_AMPHORAE_FROM_LB = 'get-amphorae-from-lb'
+HANDLE_NETWORK_DELTA = 'handle-network-delta'
+MARK_AMPHORA_DELETED = 'mark-amphora-deleted'
+MARK_AMPHORA_PENDING_DELETE = 'mark-amphora-pending-delete'
+MARK_AMPHORA_HEALTH_BUSY = 'mark-amphora-health-busy'
 RELOAD_AMP_AFTER_PLUG_VIP = 'reload-amp-after-plug-vip'
 RELOAD_LB_AFTER_AMP_ASSOC = 'reload-lb-after-amp-assoc'
 RELOAD_LB_AFTER_AMP_ASSOC_FULL_GRAPH = 'reload-lb-after-amp-assoc-full-graph'
 RELOAD_LB_AFTER_PLUG_VIP = 'reload-lb-after-plug-vip'
-RELOAD_LB_BEFOR_ALLOCATE_VIP = "reload-lb-before-allocate-vip"
+RELOAD_LB_BEFOR_ALLOCATE_VIP = 'reload-lb-before-allocate-vip'
+UPDATE_AMP_FAILOVER_DETAILS = 'update-amp-failover-details'
+
 
 NOVA_1 = '1.1'
 NOVA_21 = '2.1'
@@ -724,3 +777,13 @@ SUPPORTED_VOLUME_DRIVERS = [VOLUME_NOOP_DRIVER,
 CINDER_STATUS_AVAILABLE = 'available'
 CINDER_STATUS_ERROR = 'error'
 CINDER_ACTION_CREATE_VOLUME = 'create volume'
+
+VIP_SECURITY_GROUP_PREFIX = 'lb-'
+
+AMP_BASE_PORT_PREFIX = 'octavia-lb-vrrp-'
+OCTAVIA_OWNED = 'octavia_owned'
+
+# Sadly in the LBaaS v2 API, header insertions are on the listener objects
+# but they should be on the pool. Dealing with it until v3.
+LISTENER_PROTOCOLS_SUPPORTING_HEADER_INSERTION = [PROTOCOL_HTTP,
+                                                  PROTOCOL_TERMINATED_HTTPS]
